function sp=stdsplot(a,b,o,k,c,cov,r,t)
% sp=stdsplot(a,b,o,k,c,cov,r,t): standardizes weights, calculates effective
%                                 risk and plots weights from EPICURE results
%
% a,b: start and end point of time scale of interest
% o:   order of fitted spline
% k:   knots of fitted spline (augmented knot list has to be provided)
% c:   estimated coefficients of fitted spline
% cov: estimated covariance matrix of coefficients of fitted spline
% r:   risk type, err = excess additive risk, or = odds ratio
% t:   time scale, tse = time since exposure, aae = age at exposure

% check consistency of input arguments
if length(k)-o~=length(c)
   error('# knots - order must equal # coefficients')
end
if size(cov,1)~=size(cov,2)
   error('covariance matrix must be quadratic')
end
if size(cov,1)~=length(c)
   error('length of coefficient vector and size of covariance matrix do not match')
end

% use coefficient vector as line vector
if size(c,1)<size(c,2)
   c=c';
end

% use knot list as column vector
if size(k,1)>size(k,2)
   k=k';
end

% assign proper x labels
switch t
   case 'tse'
      xl=['Time since exposure (years)'];   
   case 'aae'
      xl=['Age at exposure (years)'];   
   otherwise
      error('Time scale (8th input argument) must be tse or aae.');
end         

% assign proper y labels
switch r
   case 'or'
      yl=['OR'];   
   case 'err'
      yl=['ERR'];   
   otherwise
      error('Risk type (7th input argument) must be err or or.');
end      

% create collocation matrix for spline
B=spcol(k,o,a:b);

% create spline
s=B*c;

% calculate variance of spline function
part1=0;
part2=0;
m=length(k)-o;
for i=1:m
   part1=part1+B(:,i).^2*cov(i,i);
   for j=1:i-1
      part2=part2+2*B(:,i).*B(:,j)*cov(i,j);
   end
end
var_s=part1+part2;

% calculate variance of spline function using matrix notation
testvar_s=diag(B*cov*B');

% calculate beta of effective (i.e. weighted) exposure
beta=ones(1,b-a+1)*B*c/(b-a+1);

% calculate variance of beta of effective (i.e. weighted) exposure
var_beta=ones(1,b-a+1)*B*cov*B'*ones(b-a+1,1)/(b-a+1);

% standardize estimated yearly risks to weights that sum up to (b-a+1)
w=(B*c)./beta;

% calculate variance for yearly weights
% 1 var((s(t))
% already done

% 2 var(sum(s(t)))
part1=0;
part2=0;
part1=sum(var_s);
for t1=a:b
   for t2=a:t1-1
      for i=1:m
         for j=1:m
            part2=part2+2*B(t1-a+1,i)*B(t2-a+1,j)*cov(i,j);
%            part2=part2+2*B(t1,i)*B(t2,j)*cov(i,j);
         end
      end
   end
end
var_sums=part1+part2;

% 3 cov(s(t),sum(s(t)))
cov_ssums=0;
for t=a:b
   for i=1:m
      for j=1:m
         cov_ssums=cov_ssums+B(t-a+1,i)*B(t-a+1,j)*cov(i,j);
%         cov_ssums=cov_ssums+B(t,i)*B(t,j)*cov(i,j);
      end
   end
end

% 4 var(w(t))
var_w=(b-a+1)^2*(var_s/sum(s)^2+s.^2*var_sums/sum(s)^4-2*s.*cov_ssums/sum(s)^3);

% calculate variance for yearly weights using matrix notation
%var_w=diag(B*cov*B')/beta^2 ...
%   +(B*c).^2*(ones(1,b-a+1)*B*cov*B'*ones(b-a+1,1)/(b-a+1)^2)/beta^4 ...
%   -2*(B*c).*(B*cov*B'*ones(b-a+1,1)/(b-a+1))/beta^3;

% prepare for results package
switch r
   case 'or'
      or_95ci=[exp(beta) exp(beta-1.96*sqrt(var_beta)) exp(beta+1.96*sqrt(var_beta))];
      sp.or_95ci=or_95ci;
      ort_95ci=[(a:b)' exp(s) sqrt(var_s) exp(s-1.96*sqrt(var_s)) exp(s+1.96*sqrt(var_s))];
      sp.ort_95ci=ort_95ci;
   case 'err'
      err_95ci=[beta beta-1.96*sqrt(var_beta) beta+1.96*sqrt(var_beta)];
      sp.err_95ci=err_95ci;
      errt_95ci=[(a:b)' s sqrt(var_s) s-1.96*sqrt(var_s) s+1.96*sqrt(var_s)];
      sp.errt_95ci=errt_95ci;
end      
w_95ci=[(a:b)' w sqrt(var_w) w-1.96*sqrt(var_w) w+1.96*sqrt(var_w)];

% results package
sp.title='Standardized weights and risk of effective exposure based on EPICURE results';
sp.interval=[a,b];
sp.scale=t;
sp.order=o;
sp.knots=k;
sp.coeffs=c;
sp.cov=cov;
sp.w_95ci=w_95ci;

% first, plot un-standardized weights (i.e. yearly OR or ERR)
switch r
   case 'or'
%      figure('name',['Yearly OR from estimated spline with pointwise 95% CI']);
      plot(ort_95ci(:,1),ort_95ci(:,2),'-',ort_95ci(:,1),ort_95ci(:,4),'--',ort_95ci(:,1),ort_95ci(:,5),'--');
%      plot((a:b)',exp(s),'-',(a:b)',exp(s-1.96*sqrt(var_s),'--',(a:b)',exp(s-1.96*sqrt(var_s),'--');
      hold on
         % plot reference line at 1
         plot((a:b)',ones(b-a+1,1),'k:');
         % plot knots
         plot(k(o:end-o+1)',ones(length(k)-2*(o-1),1),'ro');
   case 'err'
%      figure('name',['Yearly ERR from estimated spline with pointwise 95% CI']);
      plot(errt_95ci(:,1),errt_95ci(:,2),'-',errt_95ci(:,1),errt_95ci(:,4),'--',errt_95ci(:,1),errt_95ci(:,5),'--');
%      plot((a:b)',s,'-',(a:b)',s-1.96*sqrt(var_s),'--',(a:b)',s+1.96*sqrt(var_s),'--');
      hold on
         % plot reference line at 0
         plot((a:b)',zeros(b-a+1,1),'k:');
         % plot knots
         plot(k(o:end-o+1)',zeros(length(k)-2*(o-1),1),'ro');
end      
         xlabel(xl,'VerticalAlignment','Top','FontSize',16);   
         ylabel(yl,'VerticalAlignment','Bottom','FontSize',16);   
         set(gca,'FontSize',16);
     hold off         

% now plot standardized results
%figure('name',['Estimated spline weights with pointwise 95% CI']);
%plot(w_95ci(:,1),w_95ci(:,2),'-',w_95ci(:,1),w_95ci(:,4),'--',w_95ci(:,1),w_95ci(:,5),'--');
%hold on
%   % plot reference line at 1
%   plot(w_95ci(:,1),ones(b-a+1,1),'k--');
%   % plot knots
%   plot(k(o:end-o+1)',ones(length(k)-2*(o-1),1),'ro');
%   xlabel(xl,'VerticalAlignment','Top','FontSize',16);   
%   ylabel('Weights','VerticalAlignment','Bottom','FontSize',16);   
%   set(gca,'FontSize',16);
%hold off;